@@ -13,17 +13,20 @@ use etwin_core::hammerfest::{
   HammerfestShop, HammerfestShopResponse, HammerfestStore, HammerfestUserId, HammerfestUserIdRef, HammerfestUsername,
   ShortHammerfestUser, StoredHammerfestUser,
 };
+use etwin_core::pg_num::{PgU32, PgU8};
 use etwin_core::types::EtwinError;
 use etwin_core::uuid::UuidGenerator;
 use etwin_populate::hammerfest::populate_hammerfest;
 use etwin_postgres_tools::upsert_archive_query;
+use serde::Deserialize;
 use sha3::{Digest, Sha3_256};
 use sqlx::postgres::PgQueryResult;
 use sqlx::types::Uuid;
 use sqlx::{PgPool, Postgres, Transaction};
 use std::collections::{BTreeMap, BTreeSet, HashMap, HashSet};
-use std::convert::{TryFrom, TryInto};
+use std::convert::TryInto;
 use std::error::Error;
+use std::fs;
 use std::num::NonZeroU16;
 
 pub struct PgHammerfestStore<TyClock, TyDatabase, TyUuidGenerator>
@@ -57,6 +60,40 @@ where
     let mut tx = database.as_ref().begin().await.map_err(box_sqlx_error)?;
     populate_hammerfest(&mut tx).await?;
     tx.commit().await.map_err(box_sqlx_error)?;
+    let hammerfest_archive = fs::read("../../hammerfest.json");
+    dbg!(hammerfest_archive.is_err());
+    if let Ok(hammerfest_archive) = hammerfest_archive {
+      let mut tx = database.as_ref().begin().await.map_err(box_sqlx_error)?;
+      #[derive(Deserialize)]
+      struct HfUser {
+        updated_at: Instant,
+        server: HammerfestServer,
+        id: u32,
+        username: HammerfestUsername,
+        // email: Option<EmailAddress>,
+        // best_score: u32,
+        // best_level: u8,
+        // game_completed: bool,
+        items: HashMap<HammerfestItemId, u32>,
+      }
+      let hammerfest_archive: Vec<HfUser> = serde_json::from_slice(&hammerfest_archive).unwrap();
+      dbg!(hammerfest_archive.len());
+      for u in hammerfest_archive.into_iter() {
+        let short = ShortHammerfestUser {
+          server: u.server,
+          id: u.id.to_string().as_str().parse().unwrap(),
+          username: u.username,
+        };
+        touch_hammerfest_user(&mut tx, u.updated_at, &short).await.unwrap();
+        let items = touch_hammerfest_item_counts(&mut tx, &u.items, uuid_generator.next())
+          .await
+          .unwrap();
+        touch_hammerfest_inventory(&mut tx, u.updated_at, short.as_ref(), items)
+          .await
+          .unwrap();
+      }
+      tx.commit().await.map_err(box_sqlx_error)?;
+    }
     Ok(Self {
       clock,
       database,
@@ -646,7 +683,7 @@ async fn touch_hammerfest_item_counts(
       )
       .bind(map_id)
       .bind(id)
-      .bind(i32::try_from(*count).unwrap()) // TODO: Handle overflow
+      .bind(PgU32::from(*count))
       .execute(&mut *tx)
       .await?;
       assert_eq!(res.rows_affected(), 1);
@@ -656,7 +693,7 @@ async fn touch_hammerfest_item_counts(
     #[derive(Debug, sqlx::FromRow)]
     struct Row {
       hammerfest_item_id: HammerfestItemId,
-      count: u32,
+      count: PgU32,
     }
 
     let rows: Vec<Row> = sqlx::query_as::<_, Row>(
@@ -670,7 +707,10 @@ async fn touch_hammerfest_item_counts(
     .fetch_all(&mut *tx)
     .await?;
 
-    let actual: BTreeMap<HammerfestItemId, u32> = rows.iter().map(|r| (r.hammerfest_item_id, r.count)).collect();
+    let actual: BTreeMap<HammerfestItemId, u32> = rows
+      .iter()
+      .map(|r| (r.hammerfest_item_id, u32::from(r.count)))
+      .collect();
 
     assert_eq!(actual, sorted);
   }
@@ -710,9 +750,9 @@ async fn touch_hammerfest_profile(
     .bind(now)
     .bind(user.server)
     .bind(user.id)
-    .bind(best_score)
-    .bind(i16::from(best_level))
-    .bind(season_score)
+    .bind(PgU32::from(best_score))
+    .bind(PgU8::from(best_level))
+    .bind(PgU32::from(season_score))
     .bind(quests)
     .bind(unlocked_items)
     .execute(&mut *tx)
@@ -802,7 +842,7 @@ async fn touch_hammerfest_best_season_rank(
   .bind(now)
   .bind(user.server)
   .bind(user.id)
-  .bind(season_rank.map(i64::from))
+  .bind(season_rank.map(PgU32::from))
   .execute(&mut *tx)
   .await?;
   // Affected row counts:
@@ -916,7 +956,7 @@ async fn touch_hammerfest_tokens(
   .bind(now)
   .bind(user.server)
   .bind(user.id)
-  .bind(i64::from(tokens))
+  .bind(PgU32::from(tokens))
   .execute(&mut *tx)
   .await?;
   // Affected row counts:
@@ -944,7 +984,7 @@ async fn touch_hammerfest_godchild_list(
   .bind(now)
   .bind(godfather.server)
   .bind(godfather.id)
-  .bind(i64::from(godchild_count))
+  .bind(PgU32::from(godchild_count))
   .execute(&mut *tx)
   .await?;
   // Affected row counts:
@@ -976,9 +1016,9 @@ async fn touch_hammerfest_godchild(
   .bind(now)
   .bind(godfather.server)
   .bind(godfather.id)
-  .bind(i64::from(offset_in_list))
+  .bind(PgU32::from(offset_in_list))
   .bind(godchild.id)
-  .bind(i64::from(tokens))
+  .bind(PgU32::from(tokens))
   .execute(&mut *tx)
   .await?;
   // Affected row counts:
@@ -1076,24 +1116,30 @@ where
   async fn touch_shop(&self, response: &HammerfestShopResponse) -> Result<(), EtwinError> {
     let now = self.clock.now();
     let mut tx = self.database.as_ref().begin().await?;
-    touch_hammerfest_session_user(&mut tx, now, &response.session_user).await?;
-    touch_hammerfest_shop(&mut tx, now, response.session_user.user.as_ref(), &response.shop).await?;
+    touch_hammerfest_session_user(&mut tx, now, &response.session).await?;
+    touch_hammerfest_shop(&mut tx, now, response.session.user.as_ref(), &response.shop).await?;
     tx.commit().await?;
     Ok(())
   }
 
   async fn touch_profile(&self, response: &HammerfestProfileResponse) -> Result<(), EtwinError> {
+    let profile = if let Some(profile) = response.profile.as_ref() {
+      profile
+    } else {
+      return Ok(());
+    };
+
     assert_eq!(
-      response.session_user.is_some(),
-      response.profile.email.is_some(),
+      response.session.is_some(),
+      profile.email.is_some(),
       "session user presence must match email knowledge presence"
     );
     let now = self.clock.now();
     let mut tx = self.database.as_ref().begin().await?;
-    if let Some(session_user) = response.session_user.as_ref() {
+    if let Some(session_user) = response.session.as_ref() {
       touch_hammerfest_session_user(&mut tx, now, &session_user).await?;
     }
-    let options = &response.profile;
+    let options = profile;
     touch_hammerfest_user(&mut tx, now, &options.user).await?;
     let quests = touch_hammerfest_quest_statuses(&mut tx, &options.quests, self.uuid_generator.next()).await?;
     let unlocked_items = touch_hammerfest_unlocked_items(&mut tx, &options.items, self.uuid_generator.next()).await?;
@@ -1134,10 +1180,10 @@ where
   async fn touch_inventory(&self, response: &HammerfestInventoryResponse) -> Result<(), EtwinError> {
     let now = self.clock.now();
     let mut tx = self.database.as_ref().begin().await?;
-    touch_hammerfest_session_user(&mut tx, now, &response.session_user).await?;
+    touch_hammerfest_session_user(&mut tx, now, &response.session).await?;
     let inventory = &response.inventory;
     let items = touch_hammerfest_item_counts(&mut tx, inventory, self.uuid_generator.next()).await?;
-    touch_hammerfest_inventory(&mut tx, now, response.session_user.user.as_ref(), items).await?;
+    touch_hammerfest_inventory(&mut tx, now, response.session.user.as_ref(), items).await?;
     tx.commit().await?;
 
     Ok(())
@@ -1146,11 +1192,11 @@ where
   async fn touch_godchildren(&self, response: &HammerfestGodchildrenResponse) -> Result<(), EtwinError> {
     let now = self.clock.now();
     let mut tx = self.database.as_ref().begin().await?;
-    touch_hammerfest_session_user(&mut tx, now, &response.session_user).await?;
+    touch_hammerfest_session_user(&mut tx, now, &response.session).await?;
     touch_hammerfest_godchild_list(
       &mut tx,
       now,
-      response.session_user.user.as_ref(),
+      response.session.user.as_ref(),
       response.godchildren.len().try_into().expect("OverflowOnGodchildCount"),
     )
     .await?;
@@ -1159,7 +1205,7 @@ where
       touch_hammerfest_godchild(
         &mut tx,
         now,
-        response.session_user.user.as_ref(),
+        response.session.user.as_ref(),
         offset_in_list.try_into().expect("OverflowOnGodchildOffest"),
         godchild.user.as_ref(),
         godchild.tokens,
@@ -1174,7 +1220,7 @@ where
   async fn touch_theme_page(&self, response: &HammerfestForumThemePageResponse) -> Result<(), EtwinError> {
     let now = self.clock.now();
     let mut tx = self.database.as_ref().begin().await?;
-    if let Some(session_user) = response.session_user.as_ref() {
+    if let Some(session_user) = response.session.as_ref() {
       touch_hammerfest_session_user(&mut tx, now, &session_user).await?;
     }
     let options = &response.page;
@@ -1297,7 +1343,7 @@ where
     let now = self.clock.now();
     let mut tx = self.database.as_ref().begin().await?;
     let mut session_user_is_moderator = false;
-    if let Some(session_user) = response.session_user.as_ref() {
+    if let Some(session_user) = response.session.as_ref() {
       touch_hammerfest_session_user(&mut tx, now, &session_user).await?;
     }
     let options = &response.page;
@@ -1360,7 +1406,7 @@ where
         touch_hammerfest_forum_post_id(&mut tx, now, options.thread.as_ref(), options.posts.page1, offset, mid).await?;
       }
     }
-    if let Some(session_user) = response.session_user.as_ref() {
+    if let Some(session_user) = response.session.as_ref() {
       touch_hammerfest_forum_role(
         &mut tx,
         now,
